from typing import SupportsFloat
from ._numeric import AbstractBaseNumericTag

class AbstractBaseFloatTag(AbstractBaseNumericTag):
    def __init__(self, value: SupportsFloat = 0): ...
    @property
    def py_float(self) -> float: ...

class FloatTag(AbstractBaseFloatTag): ...
class DoubleTag(AbstractBaseFloatTag): ...
