from ._value import AbstractBaseMutableTag
import numpy

class AbstractBaseArrayTag(AbstractBaseMutableTag):
    def __init__(self, value=()): ...
    def __getitem__(self, item): ...
    def __setitem__(self, item): ...
    def __array__(self, dtype=None): ...
    def __len__(self): ...
    @property
    def np_array(self) -> numpy.ndarray: ...

class ByteArrayTag(AbstractBaseArrayTag): ...
class IntArrayTag(AbstractBaseArrayTag): ...
class LongArrayTag(AbstractBaseArrayTag): ...
