cdef str CommaNewline
cdef str CommaSpace

cdef char ID_END
cdef char ID_BYTE
cdef char ID_SHORT
cdef char ID_INT
cdef char ID_LONG
cdef char ID_FLOAT
cdef char ID_DOUBLE
cdef char ID_BYTE_ARRAY
cdef char ID_STRING
cdef char ID_LIST
cdef char ID_COMPOUND
cdef char ID_INT_ARRAY
cdef char ID_LONG_ARRAY
cdef char ID_MAX
