cdef str CommaNewline = ",\n"
cdef str CommaSpace = ", "

cdef char ID_END = 0
cdef char ID_BYTE = 1
cdef char ID_SHORT = 2
cdef char ID_INT = 3
cdef char ID_LONG = 4
cdef char ID_FLOAT = 5
cdef char ID_DOUBLE = 6
cdef char ID_BYTE_ARRAY = 7
cdef char ID_STRING = 8
cdef char ID_LIST = 9
cdef char ID_COMPOUND = 10
cdef char ID_INT_ARRAY = 11
cdef char ID_LONG_ARRAY = 12
cdef char ID_MAX = 12
